"""
ServiceBridge SDK v2 — New Enterprise Session Protocol

Implements the Channel-based bidi stream session with:
- 8-state FSM lifecycle  
- Adaptive heartbeat (EWMA RTT)
- Credit-based flow control with dynamic window sizing
- Bidirectional selective replay on reconnect
- ConfigPush dynamic transport config
- Epoch fencing / GoawaySignal handling
- Symmetric with Go and Node SDKs
"""

from __future__ import annotations

import asyncio
import logging
import random
from collections import deque
from dataclasses import dataclass, field
from enum import Enum, auto
from typing import Dict, List, Optional

logger = logging.getLogger(__name__)


class SessionStateV2(Enum):
    """Enterprise 8-state FSM states."""
    CONNECTING = auto()
    HANDSHAKING = auto()
    READY = auto()
    ACTIVE = auto()
    SUSPENDED = auto()
    DRAINING = auto()
    FENCED = auto()
    CLOSED = auto()


class TransportMode(str, Enum):
    """Transport mode for worker connections."""
    DIRECT = "direct"
    PROXY = "proxy"


class FallbackPolicy(str, Enum):
    """Fallback policy when primary transport fails."""
    NONE = "none"
    FALLBACK_TO_PROXY = "fallback_to_proxy"
    FALLBACK_TO_DIRECT = "fallback_to_direct"
    FALLBACK_BEST_EFFORT = "fallback_best_effort"


@dataclass
class V2Config:
    """Configuration for the v2 worker session client.

    Symmetric with Go (V2Config) and Node (V2Config) SDKs.
    """
    server_address: str
    instance_id: str
    zone: str = ""
    transport_mode: TransportMode = TransportMode.DIRECT
    max_inflight: int = 128
    min_inflight: int = 1
    sdk_version: str = "python-v2.0.0"
    capabilities: List[str] = field(default_factory=list)
    tls_enabled: bool = False
    cert_pem: str = ""
    key_pem: str = ""
    ca_pem: str = ""

    def validate(self) -> None:
        """Validate the config. Raises ValueError on invalid values."""
        if not self.server_address:
            raise ValueError("servicebridge: server_address is required")
        if not self.instance_id:
            raise ValueError("servicebridge: instance_id is required")


@dataclass
class ResumeState:
    """Resume state for reconnect with bidirectional selective replay.

    Symmetric with Go (ResumeState) and Node (ResumeState) SDKs.
    """
    resume_token: str = ""
    epoch: int = 0
    last_received_seq: int = 0
    last_sent_seq: int = 0
    completed_command_ids: List[str] = field(default_factory=list)


@dataclass
class HelloAckV2:
    """HelloAck received from server on session establishment."""
    session_id: str
    resume_token: str
    epoch: int
    resumed: bool
    resume_from_seq: int
    replayed_commands: int
    reconciled_results: int
    heartbeat_interval_ms: int
    heartbeat_timeout_ms: int
    initial_permits: int
    max_permits: int
    effective_transport_mode: TransportMode
    resume_fail_reason: Optional[str] = None


@dataclass
class FunctionDeclarationV2:
    """Declares a function for registration (v2 Reconcile API)."""
    fn_name: str
    weight: int = 100
    input_schema_json: str = ""
    output_schema_json: str = ""
    allowed_callers: List[str] = field(default_factory=list)


@dataclass
class ConsumerGroupDeclarationV2:
    """Declares a consumer group for registration (v2 Reconcile API)."""
    name: str
    pattern: str
    mode: str = ""
    retry_policy_json: str = ""
    active: bool = True
    filter_expr: str = ""
    weight: int = 100


@dataclass
class HttpRouteDeclarationV2:
    """Declares an HTTP route for registration (v2 Reconcile API)."""
    method: str
    route_pattern: str
    request_schema_json: str = ""
    response_schema_json: str = ""
    allowed_callers: List[str] = field(default_factory=list)


@dataclass
class WorkflowDeclarationV2:
    """Declares a workflow for registration (v2 Reconcile API)."""
    name: str
    definition: str
    opts: str = ""


@dataclass
class JobDeclarationV2:
    """Declares a cron job for registration (v2 Reconcile API)."""
    cron_expr: str
    timezone: str = ""
    misfire_policy: str = ""
    target_type: str = ""
    target_ref: str = ""
    delay_ms: int = 0
    retry_policy_json: str = ""


@dataclass
class ReconcileRequestV2:
    """Declarative worker registration request (v2 Reconcile API).

    Symmetric with Go (ReconcileRequestV2) and Node (ReconcileRequestV2) SDKs.
    """
    instance_id: str
    endpoint: str = ""
    transport: str = ""
    zone: str = ""
    transport_mode: TransportMode = TransportMode.DIRECT
    functions: List[FunctionDeclarationV2] = field(default_factory=list)
    consumer_groups: List[ConsumerGroupDeclarationV2] = field(default_factory=list)
    http_routes: List[HttpRouteDeclarationV2] = field(default_factory=list)
    workflows: List[WorkflowDeclarationV2] = field(default_factory=list)
    jobs: List[JobDeclarationV2] = field(default_factory=list)
    cpu_percent: float = 0.0
    ram_mb: int = 0


@dataclass
class CircuitBreakerConfigV2:
    """Circuit breaker configuration."""
    failure_threshold: int = 5
    success_threshold: int = 3
    open_duration_ms: int = 30000
    half_open_max_requests: int = 2


@dataclass
class ZoneConfigV2:
    """Zone-aware routing configuration."""
    local_zone: str = ""
    prefer_zone: bool = False
    zone_weight_boost: int = 3


@dataclass
class ServiceTransportOverride:
    """Per-service transport config override."""
    mode: Optional[TransportMode] = None
    fallback_policy: Optional[FallbackPolicy] = None
    direct_timeout_ms: int = 0
    lb_policy: str = ""
    circuit_breaker: Optional[CircuitBreakerConfigV2] = None


@dataclass
class FunctionTransportOverride:
    """Per-function transport config override."""
    mode: Optional[TransportMode] = None
    fallback_policy: Optional[FallbackPolicy] = None
    timeout_ms: int = 0


@dataclass
class TransportConfigV2:
    """Dynamic transport configuration pushed from server via ConfigPush."""
    default_mode: Optional[TransportMode] = None
    service_overrides: Dict[str, ServiceTransportOverride] = field(default_factory=dict)
    function_overrides: Dict[str, FunctionTransportOverride] = field(default_factory=dict)
    default_circuit_breaker: Optional[CircuitBreakerConfigV2] = None
    zone: Optional[ZoneConfigV2] = None


@dataclass
class IssueCertificateRequestV2:
    """Certificate issuance request (auto mTLS).

    Symmetric with Go (IssueCertificateRequestV2) and Node (IssueCertificateRequestV2) SDKs.
    """
    instance_id: str
    csr_pem: str
    advertise_address: str = ""
    extra_sans: List[str] = field(default_factory=list)


@dataclass
class IssueCertificateResponseV2:
    """Certificate issuance response."""
    cert_pem: str
    ca_pem: str
    expires_at: int  # Unix ms


class AdaptiveHeartbeatV2:
    """Adaptive heartbeat controller (EWMA RTT-based interval adjustment).

    Symmetric with Go (AdaptiveHeartbeatV2) and Node (AdaptiveHeartbeatV2) SDKs.
    """

    def __init__(self, interval_ms: int, timeout_ms: int) -> None:
        self._interval_ms = interval_ms
        self._timeout_ms = timeout_ms
        self._miss_count = 0
        self._rtt_ewma: float = 0.0
        self._alpha = 0.3

    def next_interval_ms(self) -> int:
        base = self._interval_ms / 3
        if self._miss_count > 0:
            shift = min(self._miss_count, 4)
            base = base / (2 ** shift)
        elif self._rtt_ewma > 0 and self._rtt_ewma < 50:
            base = min(base * 2, 30_000)
        return int(max(2_000, min(30_000, base)))

    def on_pong(self, rtt_ms: float) -> None:
        self._rtt_ewma = self._alpha * rtt_ms + (1 - self._alpha) * self._rtt_ewma
        self._miss_count = 0

    def on_miss(self) -> int:
        self._miss_count += 1
        return self._miss_count

    def reset_miss(self) -> None:
        self._miss_count = 0


class FlowControlStateV2:
    """Credit-based flow control state with dynamic window sizing.

    Symmetric with Go (FlowControlState) and Node (FlowControlStateV2) SDKs.
    """

    def __init__(self, initial: int, min_w: int, max_w: int) -> None:
        self._permits = initial
        self._window = initial
        self._min_w = min_w
        self._max_w = max_w
        self._lock = asyncio.Lock()

    async def try_consume(self) -> bool:
        async with self._lock:
            if self._permits <= 0:
                return False
            self._permits -= 1
            return True

    async def release(self, n: int) -> None:
        async with self._lock:
            self._permits += n

    async def available(self) -> int:
        async with self._lock:
            return self._permits

    async def set_window(self, n: int) -> None:
        async with self._lock:
            self._window = max(self._min_w, min(self._max_w, n))
            self._permits = self._window


class BackoffV2:
    """Exponential backoff with full jitter for reconnect.

    Symmetric with Go (BackoffV2) and Node (BackoffV2) SDKs.
    """

    def __init__(self) -> None:
        self._attempt = 0
        self._base_ms = 100
        self._max_ms = 30_000
        self._circuit_fails = 0

    def next_ms(self) -> float:
        self._attempt += 1
        calculated = self._base_ms * (2 ** min(self._attempt, 10))
        capped = min(calculated, self._max_ms)
        jittered = random.random() * capped
        return max(self._base_ms, jittered)

    def reset(self) -> None:
        self._attempt = 0
        self._circuit_fails = 0

    def record_fail(self) -> None:
        self._circuit_fails += 1

    def is_circuit_open(self) -> bool:
        return self._circuit_fails >= 10

    def next(self) -> float:
        """Alias for next_ms() for symmetry with Go and Node SDKs."""
        return self.next_ms()


class PositionTrackerV2:
    """Tracks sequence numbers and completed command IDs for resume.

    Symmetric with Go (PositionTracker) and Node (PositionTrackerV2) SDKs.
    """

    def __init__(self) -> None:
        self.last_received_seq: int = 0
        self.last_sent_seq: int = 0
        self._completed_ids: deque[str] = deque(maxlen=512)

    def record_received(self, seq: int) -> None:
        if seq > self.last_received_seq:
            self.last_received_seq = seq

    def record_sent(self, seq: int) -> None:
        if seq > self.last_sent_seq:
            self.last_sent_seq = seq

    def mark_completed(self, command_id: str) -> None:
        self._completed_ids.append(command_id)

    def get_resume_state(self, token: str, epoch: int) -> ResumeState:
        return ResumeState(
            resume_token=token,
            epoch=epoch,
            last_received_seq=self.last_received_seq,
            last_sent_seq=self.last_sent_seq,
            completed_command_ids=list(self._completed_ids),
        )


class ConfigPushStateV2:
    """Manages dynamically pushed transport configuration.

    Symmetric with Go (ConfigPushState) and Node (ConfigPushStateV2) SDKs.
    """

    def __init__(self) -> None:
        self._default_mode: TransportMode = TransportMode.DIRECT
        self._service_overrides: Dict[str, ServiceTransportOverride] = {}
        self._function_overrides: Dict[str, FunctionTransportOverride] = {}

    def apply(self, config: TransportConfigV2) -> None:
        if config.default_mode is not None:
            self._default_mode = config.default_mode
        self._service_overrides.update(config.service_overrides)
        self._function_overrides.update(config.function_overrides)

    def resolve_mode(self, fn_name: str) -> TransportMode:
        if fn_name in self._function_overrides:
            override = self._function_overrides[fn_name]
            if override.mode is not None:
                return override.mode
        svc_name = fn_name.split("/")[0] if "/" in fn_name else fn_name
        if svc_name in self._service_overrides:
            override = self._service_overrides[svc_name]
            if override.mode is not None:
                return override.mode
        return self._default_mode


class V2SessionClient:
    """Worker session client implementing the new Enterprise protocol.

    Symmetric with Go (V2SessionClient) and Node (V2SessionClient) SDKs.
    """

    def __init__(self, config: V2Config) -> None:
        config.validate()
        self.config = config
        self._state = SessionStateV2.CONNECTING
        self._resume_token = ""
        self._epoch = 0
        self._lock = asyncio.Lock()

        self.position = PositionTrackerV2()
        self.flow_ctrl = FlowControlStateV2(
            config.max_inflight,
            config.min_inflight,
            config.max_inflight,
        )
        self.heartbeat = AdaptiveHeartbeatV2(10_000, 30_000)
        self.backoff = BackoffV2()
        self.config_push = ConfigPushStateV2()

        self._draining = False
        self._stopped = False

    @property
    def state(self) -> SessionStateV2:
        return self._state

    @property
    def is_draining(self) -> bool:
        return self._draining

    @property
    def is_stopped(self) -> bool:
        return self._stopped

    def get_hello_fields(self) -> dict:
        """Returns Hello fields for initial connection or resume.

        Symmetric with Go (GetResumeState) and Node (getHelloFields) SDKs.
        """
        rs = self.position.get_resume_state(self._resume_token, self._epoch)
        return {
            "identity": {
                "instance_id": self.config.instance_id,
                "transport": self.config.transport_mode.value,
            },
            "max_inflight": self.config.max_inflight,
            "min_inflight": self.config.min_inflight,
            "resume_token": rs.resume_token,
            "epoch": rs.epoch,
            "last_received_seq": rs.last_received_seq,
            "last_sent_seq": rs.last_sent_seq,
            "completed_command_ids": rs.completed_command_ids,
            "sdk_version": self.config.sdk_version,
            "capabilities": self.config.capabilities,
            "zone": self.config.zone,
            "transport_mode": self.config.transport_mode.value,
        }

    async def on_hello_ack(self, ack: HelloAckV2) -> None:
        """Process HelloAck from server."""
        async with self._lock:
            self._resume_token = ack.resume_token
            self._epoch = ack.epoch
            self._state = SessionStateV2.READY
        await self.flow_ctrl.set_window(ack.initial_permits)
        self.heartbeat = AdaptiveHeartbeatV2(ack.heartbeat_interval_ms, ack.heartbeat_timeout_ms)
        self.backoff.reset()
        logger.info(
            "session-v2: hello_ack session=%s resumed=%s epoch=%d",
            ack.session_id, ack.resumed, ack.epoch,
        )

    async def on_command_received(self, seq: int, command_id: str) -> bool:
        """Process an incoming command. Returns False if backpressure."""
        self.position.record_received(seq)
        if not await self.flow_ctrl.try_consume():
            logger.warning("session-v2: no permits for command %s, backpressure", command_id)
            return False
        async with self._lock:
            if self._state == SessionStateV2.READY:
                self._state = SessionStateV2.ACTIVE
        return True

    async def on_command_completed(self, seq: int, command_id: str) -> None:
        """Process command completion — releases 1 permit."""
        self.position.mark_completed(command_id)
        self.position.record_sent(seq)
        await self.flow_ctrl.release(1)

    async def on_permit_grant(self, additional: int) -> None:
        """Handle PermitGrant from server."""
        await self.flow_ctrl.release(additional)

    async def on_flow_control_update(self, new_window_size: int, reason: str) -> None:
        """Handle FlowControlUpdate from server."""
        await self.flow_ctrl.set_window(new_window_size)
        logger.info("session-v2: flow_control_update window=%d reason=%s", new_window_size, reason)

    def on_pong(self, rtt_ms: float) -> None:
        """Handle pong response."""
        self.heartbeat.on_pong(rtt_ms)
        if self._state == SessionStateV2.SUSPENDED:
            self._state = SessionStateV2.READY
            logger.info("session-v2: heartbeat resumed, state→READY")

    def on_heartbeat_miss(self) -> bool:
        """Handle missed pong. Returns True if state→SUSPENDED."""
        count = self.heartbeat.on_miss()
        if count >= 2 and self._state in (SessionStateV2.READY, SessionStateV2.ACTIVE):
            self._state = SessionStateV2.SUSPENDED
            logger.warning("session-v2: heartbeat miss %d, state→SUSPENDED", count)
            return True
        return False

    def on_drain(self, reason: str, deadline_ms: int) -> None:
        """Initiate graceful drain (worker-initiated shutdown)."""
        if not self._draining:
            self._draining = True
            self._state = SessionStateV2.DRAINING
            logger.info("session-v2: drain initiated reason=%s deadline=%dms", reason, deadline_ms)

    def on_goaway(self, code: str, reason: str) -> None:
        """Handle GoawaySignal from server."""
        logger.info("session-v2: goaway code=%s reason=%s", code, reason)
        if code == "GOAWAY_FENCED":
            self._state = SessionStateV2.FENCED
            self.stop()
        elif code == "GOAWAY_GRACEFUL_SHUTDOWN":
            self._state = SessionStateV2.CLOSED
        else:
            self._state = SessionStateV2.CLOSED

    def on_config_push(self, config: TransportConfigV2) -> None:
        """Apply a ConfigPush from server."""
        self.config_push.apply(config)
        logger.info("session-v2: config push applied")

    def resolve_transport_mode(self, fn_name: str) -> TransportMode:
        """Resolve the effective transport mode for a function."""
        return self.config_push.resolve_mode(fn_name)

    def stop(self) -> None:
        """Stop the session."""
        self._stopped = True
        self._state = SessionStateV2.CLOSED


__all__ = [
    "V2Config",
    "V2SessionClient",
    "SessionStateV2",
    "TransportMode",
    "FallbackPolicy",
    "ResumeState",
    "HelloAckV2",
    "ReconcileRequestV2",
    "FunctionDeclarationV2",
    "ConsumerGroupDeclarationV2",
    "HttpRouteDeclarationV2",
    "WorkflowDeclarationV2",
    "JobDeclarationV2",
    "TransportConfigV2",
    "ServiceTransportOverride",
    "FunctionTransportOverride",
    "CircuitBreakerConfigV2",
    "ZoneConfigV2",
    "IssueCertificateRequestV2",
    "IssueCertificateResponseV2",
    "AdaptiveHeartbeatV2",
    "FlowControlStateV2",
    "BackoffV2",
    "PositionTrackerV2",
    "ConfigPushStateV2",
]



"""ServiceBridge middleware for FastAPI / Starlette.

Usage::

    from fastapi import FastAPI
    from service_bridge.http.fastapi import ServiceBridgeMiddleware

    app = FastAPI()
    app.add_middleware(ServiceBridgeMiddleware, client=svc)
"""

from __future__ import annotations

import threading
from typing import TYPE_CHECKING, Optional

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import Response
from starlette.types import ASGIApp

if TYPE_CHECKING:
    from service_bridge.servicebridge import ServiceBridge

# Request-scoped key for the SDK client
_CLIENT_KEY = "servicebridge_client"
_TRACE_ID_KEY = "servicebridge_trace_id"
_SPAN_ID_KEY = "servicebridge_span_id"


def get_client(request: Request) -> Optional["ServiceBridge"]:
    """Retrieve the injected ServiceBridge client from a FastAPI request."""
    return getattr(request.state, _CLIENT_KEY, None)


class ServiceBridgeMiddleware(BaseHTTPMiddleware):
    """Starlette/FastAPI middleware that adds trace propagation and HTTP catalog registration.

    Features:
    - Reads ``traceparent`` or ``x-trace-id`` from incoming headers.
    - Sets ``x-trace-id`` on the response.
    - Injects the SDK client into ``request.state.servicebridge_client``.
    - Auto-registers routes in the ServiceBridge HTTP catalog on first request.

    Args:
        app: The ASGI application.
        client: ServiceBridge SDK client.
        exclude_paths: Path prefixes to skip tracing.
        propagate_trace_header: Write ``x-trace-id`` to response (default: True).
        auto_register: Register routes on first hit (default: True).
    """

    def __init__(
        self,
        app: ASGIApp,
        client: "ServiceBridge",
        exclude_paths: tuple[str, ...] = (),
        propagate_trace_header: bool = True,
        auto_register: bool = True,
    ) -> None:
        super().__init__(app)
        self._client = client
        self._exclude_paths = exclude_paths
        self._propagate = propagate_trace_header
        self._auto_register = auto_register
        self._registered: set[str] = set()
        self._reg_lock = threading.Lock()

    async def dispatch(self, request: Request, call_next) -> Response:
        from service_bridge.http_catalog import extract_trace_from_headers

        path = request.url.path or "/"

        for prefix in self._exclude_paths:
            if path.startswith(prefix):
                setattr(request.state, _CLIENT_KEY, self._client)
                return await call_next(request)

        headers = dict(request.headers)
        trace_id, parent_span_id = extract_trace_from_headers(headers)

        span = self._client.start_http_span(
            method=request.method,
            path=path,
            trace_id=trace_id,
            parent_span_id=parent_span_id,
        )

        setattr(request.state, _CLIENT_KEY, self._client)
        setattr(request.state, _TRACE_ID_KEY, span.trace_id)
        setattr(request.state, _SPAN_ID_KEY, span.span_id)

        status_code = 500
        try:
            response = await call_next(request)
            status_code = response.status_code
        except Exception:
            span.end(status_code=status_code)
            raise

        if self._propagate:
            response.headers["x-trace-id"] = span.trace_id

        span.end(status_code=status_code)

        if self._auto_register:
            pattern = _extract_pattern(request) or path
            key = f"{request.method}:{pattern}"
            with self._reg_lock:
                if key not in self._registered:
                    self._registered.add(key)
                    method = request.method

                    def _register(m=method, r=pattern, k=key) -> None:
                        import asyncio
                        try:
                            asyncio.run(self._client.register_http_endpoint(m, r))
                        except Exception:
                            with self._reg_lock:
                                self._registered.discard(k)

                    threading.Thread(target=_register, daemon=True).start()

        return response


def _extract_pattern(request: Request) -> str:
    """Try to get the matched route pattern from the request scope."""
    route = request.scope.get("route")
    if route is not None:
        path_fmt = getattr(route, "path", None)
        if path_fmt:
            return path_fmt
    return ""



"""ServiceBridge Python SDK."""

from service_bridge.servicebridge import (
    ServiceBridge,
    ServiceBridgeError,
    ServiceBridgeErrorSeverity,
    EventContext,
    StreamWriter,
    ScheduleOpts,
    WorkflowStep,
    WorkflowOpts,
    WatchTraceOpts,
    TraceStreamEvent,
    RpcFieldDef,
    RpcSchemaOpts,
    Options,
    WorkerTLSOpts,
    ServiceBridgeLogHandler,
    servicebridge,
    get_trace_context,
    with_trace_context,
)
from service_bridge.http_catalog import HttpSpan, extract_trace_from_headers
from service_bridge.worker import RpcContext

__all__ = [
    "ServiceBridge",
    "ServiceBridgeError",
    "ServiceBridgeErrorSeverity",
    "EventContext",
    "StreamWriter",
    "RpcContext",
    "ScheduleOpts",
    "WorkflowStep",
    "WorkflowOpts",
    "WatchTraceOpts",
    "TraceStreamEvent",
    "RpcFieldDef",
    "RpcSchemaOpts",
    "Options",
    "WorkerTLSOpts",
    "ServiceBridgeLogHandler",
    "servicebridge",
    "get_trace_context",
    "with_trace_context",
    "HttpSpan",
    "extract_trace_from_headers",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: registry/v1/registry.proto
# Protobuf Python Version: 5.27.4
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    4,
    '',
    'registry/v1/registry.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from common.v1 import types_pb2 as common_dot_v1_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aregistry/v1/registry.proto\x12\x19servicebridge.registry.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x15\x63ommon/v1/types.proto\"\x83\x01\n\x10SubscribeRequest\x12%\n\x0e\x66unction_names\x18\x01 \x03(\tR\rfunctionNames\x12%\n\x0eresponse_nonce\x18\x02 \x01(\tR\rresponseNonce\x12!\n\x0c\x65rror_detail\x18\x03 \x01(\tR\x0b\x65rrorDetail\"\xd2\x01\n\x11SubscribeResponse\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12\x14\n\x05nonce\x18\x02 \x01(\tR\x05nonce\x12\x41\n\x04\x66ull\x18\x03 \x01(\x0b\x32+.servicebridge.registry.v1.RegistrySnapshotH\x00R\x04\x66ull\x12@\n\x05\x64\x65lta\x18\x04 \x01(\x0b\x32(.servicebridge.registry.v1.RegistryDeltaH\x00R\x05\x64\x65ltaB\x08\n\x06update\"\xd8\x01\n\x10RegistrySnapshot\x12X\n\tfunctions\x18\x01 \x03(\x0b\x32:.servicebridge.registry.v1.RegistrySnapshot.FunctionsEntryR\tfunctions\x1aj\n\x0e\x46unctionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x42\n\x05value\x18\x02 \x01(\x0b\x32,.servicebridge.registry.v1.FunctionEndpointsR\x05value:\x02\x38\x01\"\xe6\x01\n\rRegistryDelta\x12O\n\x07upserts\x18\x01 \x03(\x0b\x32\x35.servicebridge.registry.v1.RegistryDelta.UpsertsEntryR\x07upserts\x12\x1a\n\x08removals\x18\x02 \x03(\tR\x08removals\x1ah\n\x0cUpsertsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x42\n\x05value\x18\x02 \x01(\x0b\x32,.servicebridge.registry.v1.FunctionEndpointsR\x05value:\x02\x38\x01\"\\\n\x11\x46unctionEndpoints\x12G\n\tendpoints\x18\x01 \x03(\x0b\x32).servicebridge.registry.v1.WorkerEndpointR\tendpoints\"\xb1\x03\n\x0eWorkerEndpoint\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\x12M\n\x0etransport_mode\x18\x02 \x01(\x0e\x32&.servicebridge.common.v1.TransportModeR\rtransportMode\x12\x1f\n\x0binstance_id\x18\x03 \x01(\tR\ninstanceId\x12\x16\n\x06weight\x18\x04 \x01(\x05R\x06weight\x12!\n\x0cservice_name\x18\x05 \x01(\tR\x0bserviceName\x12\x12\n\x04zone\x18\x06 \x01(\tR\x04zone\x12*\n\x11input_schema_json\x18\n \x01(\tR\x0finputSchemaJson\x12,\n\x12output_schema_json\x18\x0b \x01(\tR\x10outputSchemaJson\x12\'\n\x0f\x61llowed_callers\x18\x0c \x03(\tR\x0e\x61llowedCallers\x12\x41\n\x06health\x18\x14 \x01(\x0b\x32).servicebridge.registry.v1.EndpointHealthR\x06health\"\xb9\x02\n\x0e\x45ndpointHealth\x12\x1c\n\treachable\x18\x01 \x01(\x08R\treachable\x12$\n\x0elatency_p50_ms\x18\x02 \x01(\x05R\x0clatencyP50Ms\x12$\n\x0elatency_p99_ms\x18\x03 \x01(\x05R\x0clatencyP99Ms\x12\x1d\n\nerror_rate\x18\x04 \x01(\x02R\terrorRate\x12\"\n\rlast_check_at\x18\x05 \x01(\x03R\x0blastCheckAt\x12\x31\n\x14\x63onsecutive_failures\x18\x06 \x01(\x05R\x13\x63onsecutiveFailures\x12G\n\x08\x63\x62_state\x18\x07 \x01(\x0e\x32,.servicebridge.common.v1.CircuitBreakerStateR\x07\x63\x62State\"=\n\rLookupRequest\x12,\n\rfunction_name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0c\x66unctionName\"\x98\x02\n\x0eLookupResponse\x12!\n\x0cservice_name\x18\x01 \x01(\tR\x0bserviceName\x12\x17\n\x07\x66n_name\x18\x02 \x01(\tR\x06\x66nName\x12G\n\tendpoints\x18\x03 \x03(\x0b\x32).servicebridge.registry.v1.WorkerEndpointR\tendpoints\x12*\n\x11input_schema_json\x18\x04 \x01(\tR\x0finputSchemaJson\x12,\n\x12output_schema_json\x18\x05 \x01(\tR\x10outputSchemaJson\x12\'\n\x0f\x61llowed_callers\x18\x06 \x03(\tR\x0e\x61llowedCallers2\xdc\x01\n\x0fRegistryService\x12j\n\tSubscribe\x12+.servicebridge.registry.v1.SubscribeRequest\x1a,.servicebridge.registry.v1.SubscribeResponse(\x01\x30\x01\x12]\n\x06Lookup\x12(.servicebridge.registry.v1.LookupRequest\x1a).servicebridge.registry.v1.LookupResponseBGZEgithub.com/servicebridge/server-go/internal/pb/registry/v1;registryv1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'registry.v1.registry_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZEgithub.com/servicebridge/server-go/internal/pb/registry/v1;registryv1'
  _globals['_REGISTRYSNAPSHOT_FUNCTIONSENTRY']._loaded_options = None
  _globals['_REGISTRYSNAPSHOT_FUNCTIONSENTRY']._serialized_options = b'8\001'
  _globals['_REGISTRYDELTA_UPSERTSENTRY']._loaded_options = None
  _globals['_REGISTRYDELTA_UPSERTSENTRY']._serialized_options = b'8\001'
  _globals['_LOOKUPREQUEST'].fields_by_name['function_name']._loaded_options = None
  _globals['_LOOKUPREQUEST'].fields_by_name['function_name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_SUBSCRIBEREQUEST']._serialized_start=110
  _globals['_SUBSCRIBEREQUEST']._serialized_end=241
  _globals['_SUBSCRIBERESPONSE']._serialized_start=244
  _globals['_SUBSCRIBERESPONSE']._serialized_end=454
  _globals['_REGISTRYSNAPSHOT']._serialized_start=457
  _globals['_REGISTRYSNAPSHOT']._serialized_end=673
  _globals['_REGISTRYSNAPSHOT_FUNCTIONSENTRY']._serialized_start=567
  _globals['_REGISTRYSNAPSHOT_FUNCTIONSENTRY']._serialized_end=673
  _globals['_REGISTRYDELTA']._serialized_start=676
  _globals['_REGISTRYDELTA']._serialized_end=906
  _globals['_REGISTRYDELTA_UPSERTSENTRY']._serialized_start=802
  _globals['_REGISTRYDELTA_UPSERTSENTRY']._serialized_end=906
  _globals['_FUNCTIONENDPOINTS']._serialized_start=908
  _globals['_FUNCTIONENDPOINTS']._serialized_end=1000
  _globals['_WORKERENDPOINT']._serialized_start=1003
  _globals['_WORKERENDPOINT']._serialized_end=1436
  _globals['_ENDPOINTHEALTH']._serialized_start=1439
  _globals['_ENDPOINTHEALTH']._serialized_end=1752
  _globals['_LOOKUPREQUEST']._serialized_start=1754
  _globals['_LOOKUPREQUEST']._serialized_end=1815
  _globals['_LOOKUPRESPONSE']._serialized_start=1818
  _globals['_LOOKUPRESPONSE']._serialized_end=2098
  _globals['_REGISTRYSERVICE']._serialized_start=2101
  _globals['_REGISTRYSERVICE']._serialized_end=2321
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: worker/v1/worker.proto
# Protobuf Python Version: 5.27.4
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    4,
    '',
    'worker/v1/worker.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16worker/v1/worker.proto\x12\x17servicebridge.worker.v1\x1a\x1b\x62uf/validate/validate.proto\"\xd7\x01\n\rHandleRequest\x12,\n\rfunction_name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0c\x66unctionName\x12\x18\n\x07payload\x18\x02 \x01(\x0cR\x07payload\x12$\n\x0egroup_trace_id\x18\x03 \x01(\tR\x0cgroupTraceId\x12\x17\n\x07span_id\x18\x04 \x01(\tR\x06spanId\x12\x19\n\x08trace_id\x18\x05 \x01(\tR\x07traceId\x12$\n\x0eparent_span_id\x18\x06 \x01(\tR\x0cparentSpanId\"X\n\x0eHandleResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\x12\x16\n\x06output\x18\x02 \x01(\x0cR\x06output\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rror\"\x99\x03\n\x0e\x44\x65liverRequest\x12&\n\ngroup_name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\tgroupName\x12\x1d\n\x05topic\x18\x02 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x05topic\x12&\n\nmessage_id\x18\x03 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\tmessageId\x12\x18\n\x07\x61ttempt\x18\x04 \x01(\x05R\x07\x61ttempt\x12\x18\n\x07payload\x18\x05 \x01(\x0cR\x07payload\x12N\n\x07headers\x18\x06 \x03(\x0b\x32\x34.servicebridge.worker.v1.DeliverRequest.HeadersEntryR\x07headers\x12$\n\x0egroup_trace_id\x18\x07 \x01(\tR\x0cgroupTraceId\x12\x17\n\x07span_id\x18\x08 \x01(\tR\x06spanId\x12\x19\n\x08trace_id\x18\t \x01(\tR\x07traceId\x1a:\n\x0cHeadersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x84\x01\n\x0f\x44\x65liverResponse\x12\x10\n\x03\x61\x63k\x18\x01 \x01(\x08R\x03\x61\x63k\x12#\n\rreject_reason\x18\x02 \x01(\tR\x0crejectReason\x12$\n\x0eretry_after_ms\x18\x03 \x01(\x05R\x0cretryAfterMs\x12\x14\n\x05\x65rror\x18\x04 \x01(\tR\x05\x65rror2\xc8\x01\n\rWorkerHandler\x12Y\n\x06Handle\x12&.servicebridge.worker.v1.HandleRequest\x1a\'.servicebridge.worker.v1.HandleResponse\x12\\\n\x07\x44\x65liver\x12\'.servicebridge.worker.v1.DeliverRequest\x1a(.servicebridge.worker.v1.DeliverResponseBCZAgithub.com/servicebridge/server-go/internal/pb/worker/v1;workerv1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'worker.v1.worker_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/servicebridge/server-go/internal/pb/worker/v1;workerv1'
  _globals['_HANDLEREQUEST'].fields_by_name['function_name']._loaded_options = None
  _globals['_HANDLEREQUEST'].fields_by_name['function_name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_DELIVERREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_DELIVERREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_DELIVERREQUEST'].fields_by_name['group_name']._loaded_options = None
  _globals['_DELIVERREQUEST'].fields_by_name['group_name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_DELIVERREQUEST'].fields_by_name['topic']._loaded_options = None
  _globals['_DELIVERREQUEST'].fields_by_name['topic']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_DELIVERREQUEST'].fields_by_name['message_id']._loaded_options = None
  _globals['_DELIVERREQUEST'].fields_by_name['message_id']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_HANDLEREQUEST']._serialized_start=81
  _globals['_HANDLEREQUEST']._serialized_end=296
  _globals['_HANDLERESPONSE']._serialized_start=298
  _globals['_HANDLERESPONSE']._serialized_end=386
  _globals['_DELIVERREQUEST']._serialized_start=389
  _globals['_DELIVERREQUEST']._serialized_end=798
  _globals['_DELIVERREQUEST_HEADERSENTRY']._serialized_start=740
  _globals['_DELIVERREQUEST_HEADERSENTRY']._serialized_end=798
  _globals['_DELIVERRESPONSE']._serialized_start=801
  _globals['_DELIVERRESPONSE']._serialized_end=933
  _globals['_WORKERHANDLER']._serialized_start=936
  _globals['_WORKERHANDLER']._serialized_end=1136
# @@protoc_insertion_point(module_scope)



"""Unit tests for the connectr Python SDK.

All tests use mocked HTTP servers and no real gRPC connection.
"""

from __future__ import annotations

import asyncio
import json
import threading
import time
import base64
from http.server import BaseHTTPRequestHandler, HTTPServer
from typing import Any
from urllib.parse import urlparse

import pytest

from service_bridge.http_catalog import extract_trace_from_headers, _new_hex_id, HttpSpan
from service_bridge.servicebridge import (
    EventContext,
    Options,
    ServiceBridge,
    WorkerTLSOpts,
    _FnChannelEntry,
    _TLSMaterials,
    _aiter_grpc_stream,
    _parse_service_key_v2,
    _resolve_ca_pem,
)


_TEST_CA_PEM = """-----BEGIN CERTIFICATE-----
MIIBkDCCATWgAwIBAgIUYRfcgaahUqALFqup7iwPh5YPXpkwCgYIKoZIzj0EAwIw
HTEbMBkGA1UEAwwSc2VydmljZWJyaWRnZS10ZXN0MB4XDTI2MDMxMzEwNTAzMVoX
DTI3MDMxMzEwNTAzMVowHTEbMBkGA1UEAwwSc2VydmljZWJyaWRnZS10ZXN0MFkw
EwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAE7apvCGZrlwyXl9CyoEC41MWjoSVxKYJ1
5wqgt2Xy7HLhu42oe3xHi/4EHgjud/FvUvxm9d0qK1VRP5WYp7+D8KNTMFEwHQYD
VR0OBBYEFIFvIQqp9rwGGgB9fWibecpnfYvAMB8GA1UdIwQYMBaAFIFvIQqp9rwG
GgB9fWibecpnfYvAMA8GA1UdEwEB/wQFMAMBAf8wCgYIKoZIzj0EAwIDSQAwRgIh
AMRHdpu4ZsTIbJ0kRZuA4dUbcTsIe2uA9tvGVCT+4g0+AiEA9Jw8vHs3COSKGKF+
GBcG18GGjC7zhBTZTtlQROqngSI=
-----END CERTIFICATE-----
"""
_TEST_CERT_PEM = _TEST_CA_PEM
_TEST_KEY_PEM = """-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgK3yq3vNMxKGMzF1t
pXm5PB+0ns/4B69cXX4jFv3c0SOhRANCAATtqm8IZmuXDJeX0LKgQLjUxaOhJXEp
gnXnCqC3ZfLscuG7jah7fEeL/gQeCO538W9S/Gb13SorVVE/lZinv4Pw
-----END PRIVATE KEY-----
"""
_TEST_SECRET_SEGMENT = base64.urlsafe_b64encode(b"0123456789abcdef0123456789abcdef").decode().rstrip("=")
_TEST_CA_SEGMENT = base64.urlsafe_b64encode(_TEST_CA_PEM.encode()).decode().rstrip("=")
_TEST_SERVICE_KEY = f"sbv2.k1.{_TEST_SECRET_SEGMENT}.{_TEST_CA_SEGMENT}"


# ─── Helpers ──────────────────────────────────────────────────────────────────

class _MockAdminHandler(BaseHTTPRequestHandler):
    """A minimal HTTP server that records POST /api/http/register calls."""

    calls: list[dict] = []
    responses: dict[str, tuple[int, str]] = {}

    def do_POST(self):
        length = int(self.headers.get("Content-Length", 0))
        body = self.rfile.read(length)
        parsed = {}
        if body:
            try:
                parsed = json.loads(body)
            except json.JSONDecodeError:
                pass
        _MockAdminHandler.calls.append({"path": self.path, "body": parsed, "headers": dict(self.headers)})

        status, response_body = _MockAdminHandler.responses.get(self.path, (200, "{}"))
        self.send_response(status)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(response_body.encode())

    def log_message(self, format, *args):  # noqa: A002
        pass  # suppress output


def make_mock_server() -> tuple[HTTPServer, str]:
    """Start a mock HTTP server on a random port and return (server, base_url)."""
    _MockAdminHandler.calls = []
    _MockAdminHandler.responses = {}
    server = HTTPServer(("localhost", 0), _MockAdminHandler)
    port = server.server_address[1]
    thread = threading.Thread(target=server.serve_forever, daemon=True)
    thread.start()
    return server, f"http://localhost:{port}"


# ─── ServiceBridge construction ───────────────────────────────────────────────

class TestServiceBridgeInit:
    def test_default_creation(self):
        key = f"sbv2.mysvc.{_TEST_SECRET_SEGMENT}.{_TEST_CA_SEGMENT}"
        svc = ServiceBridge("localhost:14445", key)
        assert svc is not None
        assert svc._service_name == "mysvc"
        assert svc._service_key == key

    def test_with_options(self):
        opts = Options(heartbeat_interval_ms=5000)
        svc = ServiceBridge("localhost:14445", "key", opts)
        assert svc._opts.heartbeat_interval_ms == 5000

    def test_instance_id_is_unique(self):
        a = ServiceBridge("localhost:14445", "key")
        b = ServiceBridge("localhost:14445", "key")
        assert a._instance_id != b._instance_id


# ─── handle_rpc decorator ─────────────────────────────────────────────────────

class TestHandleRpc:
    def test_registers_handler(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_rpc("my.fn")
        async def handler(payload):
            return {"ok": True}

        assert "my.fn" in svc._rpc_handlers
        assert svc._rpc_handlers["my.fn"] is handler

    def test_returns_original_function(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_rpc("fn1")
        def sync_handler(payload):
            return {}

        assert svc._rpc_handlers["fn1"] is sync_handler

    def test_multiple_handlers(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_rpc("fn.a")
        async def h_a(payload): return {}

        @svc.handle_rpc("fn.b")
        async def h_b(payload): return {}

        assert "fn.a" in svc._rpc_handlers
        assert "fn.b" in svc._rpc_handlers
        assert len(svc._rpc_handlers) == 2

    def test_overwrite_handler(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_rpc("fn")
        async def h1(payload): return {"v": 1}

        @svc.handle_rpc("fn")
        async def h2(payload): return {"v": 2}

        assert svc._rpc_handlers["fn"] is h2


# ─── handle_event decorator ───────────────────────────────────────────────────

class TestHandleEvent:
    def test_registers_with_default_group(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_event("order.created")
        async def handler(payload, ec): pass

        assert "consumer.order.created" in svc._event_handlers

    def test_stores_topic(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_event("my.topic")
        async def handler(payload, ec): pass

        key = "consumer.my.topic"
        assert svc._event_handlers[key]["topic"] == "my.topic"

    def test_stores_retry_policy(self):
        svc = ServiceBridge("localhost:14445", "key")
        policy = '{"maxAttempts": 3}'

        @svc.handle_event("t", retry_policy_json=policy)
        async def handler(payload, ec): pass

        key = "consumer.t"
        assert svc._event_handlers[key]["retry_policy_json"] == policy

    def test_returns_original_function(self):
        svc = ServiceBridge("localhost:14445", "key")

        @svc.handle_event("t")
        def sync_handler(payload, ec): pass

        assert svc._event_handlers["consumer.t"]["handler"] is sync_handler


# ─── EventContext ─────────────────────────────────────────────────────────────

class TestEventContext:
    def _make_ec(self):
        retry_calls = []
        reject_calls = []
        ec = EventContext(
            "trace1",
            "span1",
            "t",
            "g",
            message_id="msg1",
            attempt=1,
            headers={},
            _retry_fn=lambda ms: retry_calls.append(ms),
            _reject_fn=lambda r: reject_calls.append(r),
        )
        return ec, retry_calls, reject_calls

    def test_retry_stores_delay(self):
        ec, retry_calls, _ = self._make_ec()
        ec.retry(500)
        assert retry_calls == [500]

    def test_retry_default_delay(self):
        ec, retry_calls, _ = self._make_ec()
        ec.retry()
        assert retry_calls == [1000]

    def test_reject_stores_reason(self):
        ec, _, reject_calls = self._make_ec()
        ec.reject("bad_payload")
        assert reject_calls == ["bad_payload"]

    def test_reject_default_reason(self):
        ec, _, reject_calls = self._make_ec()
        ec.reject()
        assert reject_calls == ["rejected_by_consumer"]


# ─── extract_trace_from_headers ───────────────────────────────────────────────

class TestExtractTrace:
    def test_w3c_traceparent(self):
        headers = {"traceparent": "00-4bf92f3577b34da6a3ce929d0e0e4736-00f067aa0ba902b7-01"}
        trace_id, parent_span_id = extract_trace_from_headers(headers)
        assert trace_id == "4bf92f3577b34da6a3ce929d0e0e4736"
        assert parent_span_id == "00f067aa0ba902b7"

    def test_x_trace_id(self):
        headers = {"x-trace-id": "my-trace-123"}
        trace_id, parent_span_id = extract_trace_from_headers(headers)
        assert trace_id == "my-trace-123"
        assert parent_span_id == ""

    def test_no_headers_generates_id(self):
        trace_id, parent_span_id = extract_trace_from_headers({})
        assert trace_id != ""
        assert len(trace_id) == 32  # 16 bytes hex
        assert parent_span_id == ""

    def test_invalid_traceparent_falls_back(self):
        headers = {"traceparent": "bad-value"}
        trace_id, _ = extract_trace_from_headers(headers)
        assert trace_id != ""

    def test_case_insensitive_headers(self):
        headers = {"X-Trace-Id": "upper-case-header"}
        trace_id, _ = extract_trace_from_headers(headers)
        assert trace_id == "upper-case-header"

    def test_traceparent_takes_priority_over_x_trace_id(self):
        headers = {
            "traceparent": "00-4bf92f3577b34da6a3ce929d0e0e4736-00f067aa0ba902b7-01",
            "x-trace-id": "should-be-ignored",
        }
        trace_id, _ = extract_trace_from_headers(headers)
        assert trace_id == "4bf92f3577b34da6a3ce929d0e0e4736"


# ─── _new_hex_id ──────────────────────────────────────────────────────────────

class TestNewHexId:
    def test_default_length(self):
        result = _new_hex_id()
        assert len(result) == 32  # 16 bytes * 2

    def test_custom_length(self):
        result = _new_hex_id(8)
        assert len(result) == 16

    def test_unique(self):
        ids = {_new_hex_id() for _ in range(100)}
        assert len(ids) == 100


# ─── HttpSpan ─────────────────────────────────────────────────────────────────

class TestHttpSpan:
    def _make_mock_client(self):
        """Return a minimal object that has _report_call."""
        calls = []

        class MockClient:
            def _report_call(self, *args, **kwargs):
                calls.append(args)

        return MockClient(), calls

    def test_start_http_span_returns_span(self):
        svc = ServiceBridge("localhost:14445", "key")
        span = svc.start_http_span("GET", "/users")
        assert span is not None
        assert span.trace_id != ""
        assert span.span_id != ""

    def test_start_http_span_propagates_trace_id(self):
        svc = ServiceBridge("localhost:14445", "key")
        span = svc.start_http_span("GET", "/users", trace_id="fixed-trace-id")
        assert span.trace_id == "fixed-trace-id"

    def test_span_end_calls_report(self):
        mock_client, calls = self._make_mock_client()
        span = HttpSpan(
            trace_id="t1",
            span_id="s1",
            _fn="http:GET:/test",
            _started_at=0,
            _client=mock_client,
            _input_buf=b"{}",
        )
        span.end(status_code=200)
        assert len(calls) == 1

    def test_span_end_success_on_2xx(self):
        mock_client, calls = self._make_mock_client()
        span = HttpSpan(
            trace_id="t", span_id="s", _fn="f", _started_at=0,
            _client=mock_client, _input_buf=b"{}",
        )
        span.end(status_code=201)
        assert calls[0][5] is True  # success=True

    def test_span_end_failure_on_5xx(self):
        mock_client, calls = self._make_mock_client()
        span = HttpSpan(
            trace_id="t", span_id="s", _fn="f", _started_at=0,
            _client=mock_client, _input_buf=b"{}",
        )
        span.end(status_code=500)
        assert calls[0][5] is False  # success=False

    def test_span_end_explicit_success_override(self):
        mock_client, calls = self._make_mock_client()
        span = HttpSpan(
            trace_id="t", span_id="s", _fn="f", _started_at=0,
            _client=mock_client, _input_buf=b"{}",
        )
        span.end(status_code=500, success=True)
        assert calls[0][5] is True


# ─── register_http_endpoint (gRPC) ────────────────────────────────────────────

class _HttpStubOk:
    def __init__(self):
        self.calls: list[Any] = []

    def RegisterHttpEndpoint(self, req, metadata=None, timeout=None):  # noqa: N802
        self.calls.append((req, metadata, timeout))
        return type("Resp", (), {"ok": True})()


class _HttpStubReject:
    def RegisterHttpEndpoint(self, req, metadata=None, timeout=None):  # noqa: N802
        return type("Resp", (), {"ok": False})()


class TestRegisterHttpEndpoint:
    async def test_requires_started_client(self):
        svc = ServiceBridge("localhost:14445", "testkey")
        with pytest.raises(Exception):
            await svc.register_http_endpoint("GET", "/users")

    async def test_sends_grpc_request_payload(self):
        svc = ServiceBridge("localhost:14445", f"sbv2.testsvc.{_TEST_SECRET_SEGMENT}.{_TEST_CA_SEGMENT}")
        stub = _HttpStubOk()
        svc._stub = stub
        svc._worker_address = "127.0.0.1:9999"
        await svc.register_http_endpoint("GET", "/users", allowed_callers=["admin-service"])
        req, _, _ = stub.calls[0]
        assert req.service_name == "testsvc"
        assert req.method == "GET"
        assert req.route_pattern == "/users"
        assert list(req.allowed_callers) == ["admin-service"]

    async def test_raises_when_runtime_rejects(self):
        svc = ServiceBridge("localhost:14445", "key")
        svc._stub = _HttpStubReject()
        with pytest.raises(Exception):
            await svc.register_http_endpoint("GET", "/protected")


class _HangUnaryCall:
    def __init__(self):
        self.cancelled = False

    def result(self, timeout=None):
        _ = timeout
        time.sleep(1.0)
        return type("Resp", (), {"success": True, "output": b"{}"})()

    def cancel(self):
        self.cancelled = True


class _HangHandle:
    def __init__(self):
        self.calls: list[_HangUnaryCall] = []

    def future(self, req, timeout=None, wait_for_ready=None):
        _ = (req, timeout, wait_for_ready)
        call = _HangUnaryCall()
        self.calls.append(call)
        return call


class _HangWorkerStub:
    def __init__(self):
        self.Handle = _HangHandle()


class TestRpcTimeoutGuard:
    @pytest.mark.asyncio
    async def test_hung_rpc_attempt_is_cancelled_and_fails_by_timeout(self):
        svc = ServiceBridge("localhost:14445", "test-key")
        svc._stub = object()  # only used as started/not-started flag in rpc()

        worker_stub = _HangWorkerStub()

        async def fake_get_fn_channel(_fn: str):
            return object(), worker_stub

        svc._get_fn_channel = fake_get_fn_channel  # type: ignore[method-assign]

        with pytest.raises(Exception) as excinfo:
            await svc.rpc("svc", "hang", {"value": 1}, retries=0, timeout_ms=20)

        assert "timeout" in str(excinfo.value).lower()
        assert len(worker_stub.Handle.calls) == 1
        assert worker_stub.Handle.calls[0].cancelled is True


class TestParityValidation:
    def test_parse_service_key_v2_requires_valid_embedded_ca(self):
        _parse_service_key_v2(_TEST_SERVICE_KEY)
        bad_key = f"sbv2.k1.{_TEST_SECRET_SEGMENT}.bm90LWEtdmFsaWQtY2VydA"
        with pytest.raises(RuntimeError, match="malformed embedded ca pem"):
            _parse_service_key_v2(bad_key)

    def test_resolve_ca_pem_rejects_invalid_explicit_ca(self):
        with pytest.raises(RuntimeError, match="control-plane ca cert"):
            _resolve_ca_pem(_TEST_SERVICE_KEY, "not-a-cert")

    def test_handle_hints_are_stored(self):
        svc = ServiceBridge("localhost:14445", _TEST_SERVICE_KEY)

        @svc.handle_rpc(
            "echo",
            timeout_ms=5000,
            retryable=True,
            concurrency=8,
        )
        async def _rpc(_payload):
            return {"ok": True}

        @svc.handle_event("orders.*", concurrency=4, prefetch=32)
        async def _event(_payload, _ctx):
            return None

        assert svc._rpc_opts["echo"]["timeout_ms"] == 5000
        assert svc._rpc_opts["echo"]["retryable"] is True
        assert svc._rpc_opts["echo"]["concurrency"] == 8
        assert svc._event_handlers["k1.orders.*"]["concurrency"] == 4
        assert svc._event_handlers["k1.orders.*"]["prefetch"] == 32

    @pytest.mark.asyncio
    async def test_serve_fails_fast_on_initial_worker_session_error(self):
        svc = ServiceBridge("localhost:14445", _TEST_SERVICE_KEY)

        @svc.handle_rpc("ping")
        async def _ping(_payload):
            return {}

        svc._worker_tls_creds = _TLSMaterials(
            ca_cert=_TEST_CA_PEM.encode(),
            client_cert=_TEST_CERT_PEM.encode(),
            client_key=_TEST_KEY_PEM.encode(),
        )

        async def fake_connect_control_plane():
            svc._stub = object()

        async def fake_start_worker_server(_host: str) -> str:
            return "127.0.0.1:7443"

        def fake_start_worker_session():
            raise RuntimeError("OpenWorkerSession initial connect failed: boom")

        sync_called = False

        async def fake_sync_registrations():
            nonlocal sync_called
            sync_called = True

        svc._connect_control_plane = fake_connect_control_plane  # type: ignore[method-assign]
        svc._start_worker_server = fake_start_worker_server  # type: ignore[method-assign]
        svc._start_worker_session = fake_start_worker_session  # type: ignore[method-assign]
        svc._sync_registrations = fake_sync_registrations  # type: ignore[method-assign]

        with pytest.raises(RuntimeError, match="OpenWorkerSession initial connect failed"):
            await svc.serve(host="localhost", instance_id="i-1", weight=7)

        assert sync_called is False
        assert svc._instance_id == "i-1"
        assert svc._registration_weight == 7

    def test_resolve_explicit_worker_tls_validates_cert_and_key(self):
        svc = ServiceBridge("localhost:14445", _TEST_SERVICE_KEY)
        with pytest.raises(RuntimeError, match="must be provided together"):
            svc._resolve_explicit_worker_tls(WorkerTLSOpts(cert=_TEST_CERT_PEM))
        with pytest.raises(RuntimeError, match="cert/key pair mismatch|invalid worker tls key"):
            svc._resolve_explicit_worker_tls(
                WorkerTLSOpts(cert=_TEST_CERT_PEM, key="not-a-key")
            )
        resolved = svc._resolve_explicit_worker_tls(
            WorkerTLSOpts(cert=_TEST_CERT_PEM, key=_TEST_KEY_PEM)
        )
        assert resolved is not None
        materials, _ = resolved
        assert materials.client_cert.startswith(b"-----BEGIN CERTIFICATE-----")

    def test_fn_channel_round_robin_selector(self):
        svc = ServiceBridge("localhost:14445", _TEST_SERVICE_KEY)
        entry = _FnChannelEntry(
            channels=["ch1", "ch2"],  # type: ignore[list-item]
            stubs=["stub1", "stub2"],
            endpoints=["a:1", "b:2"],
        )
        _, stub1 = svc._pick_fn_channel(entry)
        _, stub2 = svc._pick_fn_channel(entry)
        _, stub3 = svc._pick_fn_channel(entry)
        assert stub1 == "stub1"
        assert stub2 == "stub2"
        assert stub3 == "stub1"

    @pytest.mark.asyncio
    async def test_stream_helper_fails_on_queue_overflow(self):
        class _FastStream:
            def __iter__(self):
                for i in range(128):
                    yield i

            def cancel(self):
                return None

        loop = asyncio.get_event_loop()
        with pytest.raises(RuntimeError, match="overflow"):
            async for _ in _aiter_grpc_stream(
                _FastStream(),
                loop,
                queue_limit=1,
                overflow_error_factory=lambda: RuntimeError("overflow"),
            ):
                await asyncio.sleep(0.05)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: servicebridge.proto
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from service_bridge.pb import servicebridge_pb2 as servicebridge__pb2

GRPC_GENERATED_VERSION = '1.68.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        ' but the generated code in servicebridge_pb2_grpc.py depends on'
        f' grpcio>={GRPC_GENERATED_VERSION}.'
        ' Please upgrade your grpc module to grpcio>=' + GRPC_GENERATED_VERSION
        + ' or downgrade your generated code using grpcio-tools<=' + GRPC_VERSION,
        RuntimeWarning
    )


class ServiceBridgeStub(object):
    """ServiceBridge stub."""

    def __init__(self, channel):
        """Constructor.

Args:
    channel: A grpc.Channel.
"""
        self.RegisterFunction = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterFunction',
                request_serializer=servicebridge__pb2.RegisterFunctionRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterFunctionResponse.FromString,
                )
        self.RegisterConsumerGroup = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterConsumerGroup',
                request_serializer=servicebridge__pb2.RegisterConsumerGroupRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterConsumerGroupResponse.FromString,
                )
        self.RegisterGroupMember = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterGroupMember',
                request_serializer=servicebridge__pb2.RegisterGroupMemberRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterGroupMemberResponse.FromString,
                )
        self.Heartbeat = channel.unary_unary(
                '/servicebridge.ServiceBridge/Heartbeat',
                request_serializer=servicebridge__pb2.HeartbeatRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.HeartbeatResponse.FromString,
                )
        self.RegisterHttpEndpoint = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterHttpEndpoint',
                request_serializer=servicebridge__pb2.RegisterHttpEndpointRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterHttpEndpointResponse.FromString,
                )
        self.HeartbeatHttpEndpoint = channel.unary_unary(
                '/servicebridge.ServiceBridge/HeartbeatHttpEndpoint',
                request_serializer=servicebridge__pb2.HeartbeatHttpEndpointRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.HeartbeatHttpEndpointResponse.FromString,
                )
        self.ProvisionWorkerCertificate = channel.unary_unary(
                '/servicebridge.ServiceBridge/ProvisionWorkerCertificate',
                request_serializer=servicebridge__pb2.ProvisionWorkerCertificateRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.ProvisionWorkerCertificateResponse.FromString,
                )
        self.OpenWorkerSession = channel.stream_stream(
                '/servicebridge.ServiceBridge/OpenWorkerSession',
                request_serializer=servicebridge__pb2.WorkerSessionRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.WorkerSessionResponse.FromString,
                )
        self.RegisterWorkflow = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterWorkflow',
                request_serializer=servicebridge__pb2.RegisterWorkflowRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterWorkflowResponse.FromString,
                )
        self.ExecuteWorkflow = channel.unary_unary(
                '/servicebridge.ServiceBridge/ExecuteWorkflow',
                request_serializer=servicebridge__pb2.RunWorkflowRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RunWorkflowResponse.FromString,
                )
        self.CancelWorkflow = channel.unary_unary(
                '/servicebridge.ServiceBridge/CancelWorkflow',
                request_serializer=servicebridge__pb2.CancelWorkflowRunRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.CancelWorkflowRunResponse.FromString,
                )
        self.RegisterJob = channel.unary_unary(
                '/servicebridge.ServiceBridge/RegisterJob',
                request_serializer=servicebridge__pb2.RegisterJobRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegisterJobResponse.FromString,
                )
        self.WatchRegistry = channel.unary_stream(
                '/servicebridge.ServiceBridge/WatchRegistry',
                request_serializer=servicebridge__pb2.WatchRegistryRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RegistryMessage.FromString,
                )
        self.LookupFunction = channel.unary_unary(
                '/servicebridge.ServiceBridge/LookupFunction',
                request_serializer=servicebridge__pb2.LookupFunctionRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.LookupFunctionResponse.FromString,
                )
        self.ReportCallStart = channel.unary_unary(
                '/servicebridge.ServiceBridge/ReportCallStart',
                request_serializer=servicebridge__pb2.ReportCallStartRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.ReportCallResponse.FromString,
                )
        self.ReportCall = channel.unary_unary(
                '/servicebridge.ServiceBridge/ReportCall',
                request_serializer=servicebridge__pb2.ReportCallRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.ReportCallResponse.FromString,
                )
        self.ReportLog = channel.unary_unary(
                '/servicebridge.ServiceBridge/ReportLog',
                request_serializer=servicebridge__pb2.ReportLogRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.ReportLogResponse.FromString,
                )
        self.Publish = channel.unary_unary(
                '/servicebridge.ServiceBridge/Publish',
                request_serializer=servicebridge__pb2.PublishRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.PublishResponse.FromString,
                )
        self.AppendStream = channel.unary_unary(
                '/servicebridge.ServiceBridge/AppendStream',
                request_serializer=servicebridge__pb2.AppendStreamRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.AppendStreamResponse.FromString,
                )
        self.WatchTrace = channel.unary_stream(
                '/servicebridge.ServiceBridge/WatchTrace',
                request_serializer=servicebridge__pb2.WatchRunRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.RunStreamChunk.FromString,
                )


class ServiceBridgeServicer(object):
    """ServiceBridge servicer."""
    pass


def add_ServiceBridgeServicer_to_server(servicer, server):
    pass


class ServiceBridgeWorkerStub(object):
    """ServiceBridgeWorker stub."""

    def __init__(self, channel):
        self.Handle = channel.unary_unary(
                '/servicebridge.ServiceBridgeWorker/Handle',
                request_serializer=servicebridge__pb2.HandleRequest.SerializeToString,
                response_deserializer=servicebridge__pb2.HandleResponse.FromString,
                )


class ServiceBridgeWorkerServicer(object):
    """ServiceBridgeWorker servicer."""

    def Handle(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ServiceBridgeWorkerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Handle': grpc.unary_unary_rpc_method_handler(
            servicer.Handle,
            request_deserializer=servicebridge__pb2.HandleRequest.FromString,
            response_serializer=servicebridge__pb2.HandleResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_service_handler(
        'servicebridge.ServiceBridgeWorker', rpc_method_handlers
    ) if hasattr(grpc, 'method_service_handler') else None
    if generic_handler is not None:
        server.add_generic_rpc_handlers((generic_handler,))
    else:
        server.add_registered_method_handlers('servicebridge.ServiceBridgeWorker', rpc_method_handlers)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from controlplane.v1 import control_plane_pb2 as controlplane_dot_v1_dot_control__plane__pb2


class ControlPlaneStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Reconcile = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/Reconcile',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.ReconcileRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ReconcileResponse.FromString,
                _registered_method=True)
        self.IssueCertificate = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/IssueCertificate',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateResponse.FromString,
                _registered_method=True)
        self.Publish = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/Publish',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.PublishRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.PublishResponse.FromString,
                _registered_method=True)
        self.ExecuteWorkflow = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/ExecuteWorkflow',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowResponse.FromString,
                _registered_method=True)
        self.CancelWorkflow = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/CancelWorkflow',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowResponse.FromString,
                _registered_method=True)
        self.ProxyCall = channel.unary_unary(
                '/servicebridge.controlplane.v1.ControlPlane/ProxyCall',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.ProxyCallRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ProxyCallResponse.FromString,
                _registered_method=True)
        self.WatchTrace = channel.unary_stream(
                '/servicebridge.controlplane.v1.ControlPlane/WatchTrace',
                request_serializer=controlplane_dot_v1_dot_control__plane__pb2.WatchTraceRequest.SerializeToString,
                response_deserializer=controlplane_dot_v1_dot_control__plane__pb2.TraceStreamChunk.FromString,
                _registered_method=True)


class ControlPlaneServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Reconcile(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IssueCertificate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Publish(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteWorkflow(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CancelWorkflow(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ProxyCall(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WatchTrace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ControlPlaneServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Reconcile': grpc.unary_unary_rpc_method_handler(
                    servicer.Reconcile,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ReconcileRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.ReconcileResponse.SerializeToString,
            ),
            'IssueCertificate': grpc.unary_unary_rpc_method_handler(
                    servicer.IssueCertificate,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateResponse.SerializeToString,
            ),
            'Publish': grpc.unary_unary_rpc_method_handler(
                    servicer.Publish,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.PublishRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.PublishResponse.SerializeToString,
            ),
            'ExecuteWorkflow': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteWorkflow,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowResponse.SerializeToString,
            ),
            'CancelWorkflow': grpc.unary_unary_rpc_method_handler(
                    servicer.CancelWorkflow,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowResponse.SerializeToString,
            ),
            'ProxyCall': grpc.unary_unary_rpc_method_handler(
                    servicer.ProxyCall,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.ProxyCallRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.ProxyCallResponse.SerializeToString,
            ),
            'WatchTrace': grpc.unary_stream_rpc_method_handler(
                    servicer.WatchTrace,
                    request_deserializer=controlplane_dot_v1_dot_control__plane__pb2.WatchTraceRequest.FromString,
                    response_serializer=controlplane_dot_v1_dot_control__plane__pb2.TraceStreamChunk.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'servicebridge.controlplane.v1.ControlPlane', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('servicebridge.controlplane.v1.ControlPlane', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ControlPlane(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Reconcile(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/Reconcile',
            controlplane_dot_v1_dot_control__plane__pb2.ReconcileRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.ReconcileResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def IssueCertificate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/IssueCertificate',
            controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.IssueCertificateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Publish(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/Publish',
            controlplane_dot_v1_dot_control__plane__pb2.PublishRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.PublishResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ExecuteWorkflow(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/ExecuteWorkflow',
            controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.ExecuteWorkflowResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CancelWorkflow(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/CancelWorkflow',
            controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.CancelWorkflowResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ProxyCall(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/ProxyCall',
            controlplane_dot_v1_dot_control__plane__pb2.ProxyCallRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.ProxyCallResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def WatchTrace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/servicebridge.controlplane.v1.ControlPlane/WatchTrace',
            controlplane_dot_v1_dot_control__plane__pb2.WatchTraceRequest.SerializeToString,
            controlplane_dot_v1_dot_control__plane__pb2.TraceStreamChunk.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: common/v1/types.proto
# Protobuf Python Version: 5.27.4
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    4,
    '',
    'common/v1/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x63ommon/v1/types.proto\x12\x17servicebridge.common.v1\x1a\x1b\x62uf/validate/validate.proto\"\xa0\x01\n\x0eWorkerIdentity\x12*\n\x0cservice_name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x0bserviceName\x12(\n\x0binstance_id\x18\x02 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\ninstanceId\x12\x1a\n\x08\x65ndpoint\x18\x03 \x01(\tR\x08\x65ndpoint\x12\x1c\n\ttransport\x18\x04 \x01(\tR\ttransport\"n\n\x0fResourceMetrics\x12$\n\x0b\x63pu_percent\x18\x01 \x01(\x02H\x00R\ncpuPercent\x88\x01\x01\x12\x1a\n\x06ram_mb\x18\x02 \x01(\x05H\x01R\x05ramMb\x88\x01\x01\x42\x0e\n\x0c_cpu_percentB\t\n\x07_ram_mb\"\xd2\x01\n\x13\x46unctionDeclaration\x12 \n\x07\x66n_name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x06\x66nName\x12\x16\n\x06weight\x18\x02 \x01(\x05R\x06weight\x12*\n\x11input_schema_json\x18\x03 \x01(\tR\x0finputSchemaJson\x12,\n\x12output_schema_json\x18\x04 \x01(\tR\x10outputSchemaJson\x12\'\n\x0f\x61llowed_callers\x18\x05 \x03(\tR\x0e\x61llowedCallers\"\xeb\x01\n\x18\x43onsumerGroupDeclaration\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x04name\x12!\n\x07pattern\x18\x02 \x01(\tB\x07\xbaH\x04r\x02\x10\x01R\x07pattern\x12\x12\n\x04mode\x18\x03 \x01(\tR\x04mode\x12*\n\x11retry_policy_json\x18\x04 \x01(\tR\x0fretryPolicyJson\x12\x16\n\x06\x61\x63tive\x18\x05 \x01(\x08R\x06\x61\x63tive\x12\x1f\n\x0b\x66ilter_expr\x18\x06 \x01(\tR\nfilterExpr\x12\x16\n\x06weight\x18\x07 \x01(\x05R\x06weight\"\xde\x01\n\x14HttpRouteDeclaration\x12\x16\n\x06method\x18\x01 \x01(\tR\x06method\x12#\n\rroute_pattern\x18\x02 \x01(\tR\x0croutePattern\x12.\n\x13request_schema_json\x18\x03 \x01(\tR\x11requestSchemaJson\x12\x30\n\x14response_schema_json\x18\x04 \x01(\tR\x12responseSchemaJson\x12\'\n\x0f\x61llowed_callers\x18\x05 \x03(\tR\x0e\x61llowedCallers\"]\n\x13WorkflowDeclaration\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1e\n\ndefinition\x18\x02 \x01(\tR\ndefinition\x12\x12\n\x04opts\x18\x03 \x01(\tR\x04opts\"\xf7\x01\n\x0eJobDeclaration\x12\x1b\n\tcron_expr\x18\x01 \x01(\tR\x08\x63ronExpr\x12\x1a\n\x08timezone\x18\x02 \x01(\tR\x08timezone\x12%\n\x0emisfire_policy\x18\x03 \x01(\tR\rmisfirePolicy\x12\x1f\n\x0btarget_type\x18\x04 \x01(\tR\ntargetType\x12\x1d\n\ntarget_ref\x18\x05 \x01(\tR\ttargetRef\x12\x19\n\x08\x64\x65lay_ms\x18\x06 \x01(\x05R\x07\x64\x65layMs\x12*\n\x11retry_policy_json\x18\x07 \x01(\tR\x0fretryPolicyJson*d\n\rTransportMode\x12\x1e\n\x1aTRANSPORT_MODE_UNSPECIFIED\x10\x00\x12\x19\n\x15TRANSPORT_MODE_DIRECT\x10\x01\x12\x18\n\x14TRANSPORT_MODE_PROXY\x10\x02*g\n\x13LoadBalancingPolicy\x12\x12\n\x0eLB_ROUND_ROBIN\x10\x00\x12\x0f\n\x0bLB_WEIGHTED\x10\x01\x12\x18\n\x14LB_LEAST_CONNECTIONS\x10\x02\x12\x11\n\rLB_ZONE_AWARE\x10\x03*C\n\x13\x43ircuitBreakerState\x12\r\n\tCB_CLOSED\x10\x00\x12\x0b\n\x07\x43\x42_OPEN\x10\x01\x12\x10\n\x0c\x43\x42_HALF_OPEN\x10\x02\x42\x43ZAgithub.com/servicebridge/server-go/internal/pb/common/v1;commonv1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.v1.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/servicebridge/server-go/internal/pb/common/v1;commonv1'
  _globals['_WORKERIDENTITY'].fields_by_name['service_name']._loaded_options = None
  _globals['_WORKERIDENTITY'].fields_by_name['service_name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_WORKERIDENTITY'].fields_by_name['instance_id']._loaded_options = None
  _globals['_WORKERIDENTITY'].fields_by_name['instance_id']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_FUNCTIONDECLARATION'].fields_by_name['fn_name']._loaded_options = None
  _globals['_FUNCTIONDECLARATION'].fields_by_name['fn_name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_CONSUMERGROUPDECLARATION'].fields_by_name['name']._loaded_options = None
  _globals['_CONSUMERGROUPDECLARATION'].fields_by_name['name']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_CONSUMERGROUPDECLARATION'].fields_by_name['pattern']._loaded_options = None
  _globals['_CONSUMERGROUPDECLARATION'].fields_by_name['pattern']._serialized_options = b'\272H\004r\002\020\001'
  _globals['_TRANSPORTMODE']._serialized_start=1375
  _globals['_TRANSPORTMODE']._serialized_end=1475
  _globals['_LOADBALANCINGPOLICY']._serialized_start=1477
  _globals['_LOADBALANCINGPOLICY']._serialized_end=1580
  _globals['_CIRCUITBREAKERSTATE']._serialized_start=1582
  _globals['_CIRCUITBREAKERSTATE']._serialized_end=1649
  _globals['_WORKERIDENTITY']._serialized_start=80
  _globals['_WORKERIDENTITY']._serialized_end=240
  _globals['_RESOURCEMETRICS']._serialized_start=242
  _globals['_RESOURCEMETRICS']._serialized_end=352
  _globals['_FUNCTIONDECLARATION']._serialized_start=355
  _globals['_FUNCTIONDECLARATION']._serialized_end=565
  _globals['_CONSUMERGROUPDECLARATION']._serialized_start=568
  _globals['_CONSUMERGROUPDECLARATION']._serialized_end=803
  _globals['_HTTPROUTEDECLARATION']._serialized_start=806
  _globals['_HTTPROUTEDECLARATION']._serialized_end=1028
  _globals['_WORKFLOWDECLARATION']._serialized_start=1030
  _globals['_WORKFLOWDECLARATION']._serialized_end=1123
  _globals['_JOBDECLARATION']._serialized_start=1126
  _globals['_JOBDECLARATION']._serialized_end=1373
# @@protoc_insertion_point(module_scope)
